@@ -1,7 +1,7 @@
 /******************************************************************************
 *******************************************************************************
 **
-**  Copyright (C) 2005-2006 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2005-2008 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -15,6 +15,7 @@
 #include <netinet/in.h>
 #include <syslog.h>
 #include <string.h>
+#include <signal.h>
 #include <sys/utsname.h>
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -29,11 +30,14 @@
 #include "cnxman-private.h"
 #include "logging.h"
 #include "commands.h"
+#include "daemon.h"
 #include "ais.h"
 #include "ccs.h"
 
 
 #define DEFAULT_PORT            5405
+#define DEFAULT_CLUSTER_NAME    "RHCluster"
+#define NOCCS_KEY_FILENAME      DEFAULT_CONFIG_DIR "/cman_authkey"
 
 #define CONFIG_VERSION_PATH	"/cluster/@config_version"
 #define CLUSTER_NAME_PATH	"/cluster/@name"
@@ -44,8 +48,8 @@
 #define MCAST_ADDR_PATH		"/cluster/cman/multicast/@addr"
 #define PORT_PATH		"/cluster/cman/@port"
 #define KEY_PATH		"/cluster/cman/@keyfile"
+#define MAXQUEUED_PATH		"/cluster/cman/@max_queued"
 
-#define NODEI_NAME_PATH		"/cluster/clusternodes/clusternode[%d]/@name"
 #define NODE_NAME_PATH_BYNAME	"/cluster/clusternodes/clusternode[@name=\"%s\"]/@name"
 #define NODE_NAME_PATH_BYNUM	"/cluster/clusternodes/clusternode[%d]/@name"
 #define NODE_VOTES_PATH		"/cluster/clusternodes/clusternode[@name=\"%s\"]/@votes"
@@ -62,6 +66,7 @@ static int nodeid;
 static char *nodenames[MAX_NODENAMES];
 static int  portnums[MAX_NODENAMES];
 static char *mcast[MAX_NODENAMES];
+static char *nodename_env;
 static int num_nodenames;
        int two_node;
        char *key_filename;
@@ -71,6 +76,8 @@ static unsigned int expected_votes;
 static unsigned short cluster_id;
 static char cluster_name[MAX_CLUSTER_NAME_LEN + 1];
 
+LOGSYS_DECLARE_SUBSYS (CMAN_NAME, LOG_INFO);
+
 static uint16_t generate_cluster_id(char *name)
 {
 	int i;
@@ -99,10 +106,15 @@ int read_ccs_nodes(unsigned int *config_version, int check_nodeids)
     int expected = 0;
     unsigned int config;
 
+    if (getenv("CMAN_NOCCS")) {
+	    *config_version = 1;
+	    return 0;
+    }
+
     /* Open the config file */
     ctree = ccs_force_connect(NULL, 1);
     if (ctree < 0) {
-	    log_msg(LOG_ERR, "Error connecting to CCS");
+	    log_printf(LOG_ERR, "Error connecting to CCS");
 	    write_cman_pipe("Cannot connect to CCS");
 	    return -1;
     }
@@ -124,47 +136,60 @@ int read_ccs_nodes(unsigned int *config_version, int check_nodeids)
     if (!ccs_get(ctree, TWO_NODE_PATH, &str)) {
 	    two_node = atoi(str);
 	    free(str);
-    }
-    else
-	two_node = 0;
+    } else
+		two_node = 0;
 
     for (i=1;;i++) {
-	char nodekey[256];
-	char key[256];
-	int  votes=0, nodeid=0;
+		char path[MAX_PATH_LEN];
+		int votes=0, nodeid=0;
+		int ret;
 
-	sprintf(nodekey, NODE_NAME_PATH_BYNUM, i);
-	error = ccs_get(ctree, nodekey, &nodename);
-	if (error)
-	    break;
+		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNUM, i);
+		if (ret < 0 || (size_t) ret >= sizeof(path))
+			return -E2BIG;
 
-	sprintf(key, NODE_VOTES_PATH, nodename);
-	if (!ccs_get(ctree, key, &str)) {
-	    votes = atoi(str);
-	    free(str);
-	} else
-	    votes = 1;
+		error = ccs_get(ctree, path, &nodename);
+		if (error)
+			break;
 
-	sprintf(key, NODE_NODEID_PATH, nodename);
-	if (!ccs_get(ctree, key, &str))	{
-	    nodeid = atoi(str);
-	    free(str);
+		ret = snprintf(path, sizeof(path), NODE_VOTES_PATH, nodename);
+		if (ret < 0 || ret >= sizeof(path)) {
+			error = -E2BIG;
+			goto out_err;
+		}
 
-	}
+		if (!ccs_get(ctree, path, &str)) {
+			votes = atoi(str);
+			free(str);
+		} else
+			votes = 1;
 
-	if (check_nodeids && nodeid == 0) {
-		char message[132];
+		ret = snprintf(path, sizeof(path), NODE_NODEID_PATH, nodename);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			goto out_err;
+		}
 
-		sprintf(message, "No node ID for %s, run 'ccs_tool addnodeids' to fix", nodename);
-		log_msg(LOG_ERR, message);
-		write_cman_pipe(message);
-		return -1;
-	}
+		if (!ccs_get(ctree, path, &str)) {
+			nodeid = atoi(str);
+			free(str);
+		}
 
-	P_MEMB("Got node %s from ccs (id=%d, votes=%d)\n", nodename, nodeid, votes);
-	add_ccs_node(nodename, nodeid, votes, expected);
+		if (check_nodeids && nodeid == 0) {
+			char message[132];
 
-	free(nodename);
+			snprintf(message, sizeof(message),
+				"No node ID for %s, run 'ccs_tool addnodeids' to fix",
+				nodename);
+			log_printf(LOG_ERR, "%s", message);
+			write_cman_pipe(message);
+			error = -EINVAL;
+			goto out_err;
+		}
+
+		P_MEMB("Got node %s from ccs (id=%d, votes=%d)\n", nodename, nodeid, votes);
+		add_ccs_node(nodename, nodeid, votes, expected);
+		free(nodename);
     }
 
     if (expected)
@@ -174,6 +199,11 @@ int read_ccs_nodes(unsigned int *config_version, int check_nodeids)
     ccs_disconnect(ctree);
 
     return 0;
+
+out_err:
+	free(nodename);
+	ccs_disconnect(ctree);
+	return error;
 }
 
 static char *default_mcast(uint16_t cluster_id)
@@ -190,7 +220,7 @@ static char *default_mcast(uint16_t cluster_id)
 	   default a multicast address */
         ret = getaddrinfo(nodenames[0], NULL, &ahints, &ainfo);
 	if (ret) {
-		log_msg(LOG_ERR, "Can't determine address family of nodename %s\n", nodenames[0]);
+		log_printf(LOG_ERR, "Can't determine address family of nodename %s\n", nodenames[0]);
 		write_cman_pipe("Can't determine address family of nodename");
 		return NULL;
 	}
@@ -257,11 +287,14 @@ static int verify_nodename(int cd, char *nodename)
 	struct ifaddrs *ifa, *ifa_list;
 	struct sockaddr *sa;
 	int error, i;
+	int ret;
 
 	/* nodename is either from commandline or from uname */
 	str = NULL;
-	memset(path, 0, MAX_PATH_LEN);
-	sprintf(path, NODE_NAME_PATH_BYNAME, nodename);
+
+	ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename);
+	if (ret < 0 || (size_t) ret >= sizeof(path))
+		return -E2BIG;
 
 	error = ccs_get(cd, path, &str);
 	if (!error) {
@@ -271,13 +304,14 @@ static int verify_nodename(int cd, char *nodename)
 
 	/* If nodename was from uname, try a domain-less version of it */
 	strcpy(nodename2, nodename);
-	dot = strstr(nodename2, ".");
+	dot = strchr(nodename2, '.');
 	if (dot) {
 		*dot = '\0';
 
 		str = NULL;
-		memset(path, 0, MAX_PATH_LEN);
-		sprintf(path, NODE_NAME_PATH_BYNAME, nodename2);
+		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename2);
+		if (ret < 0 || (size_t) ret >= sizeof(path))
+			return -E2BIG;
 
 		error = ccs_get(cd, path, &str);
 		if (!error) {
@@ -287,21 +321,25 @@ static int verify_nodename(int cd, char *nodename)
 		}
 	}
 
-
 	/* If nodename (from uname) is domain-less, try to match against
 	   cluster.conf names which may have domainname specified */
 	for (i = 1; ; i++) {
 		int len;
+
 		str = NULL;
-		memset(path, 0, 256);
-		sprintf(path, "/cluster/clusternodes/clusternode[%d]/@name", i);
+		ret = snprintf(path, sizeof(path),
+				"/cluster/clusternodes/clusternode[%d]/@name", i);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			break;
+		}
 
 		error = ccs_get(cd, path, &str);
 		if (error || !str)
 			break;
 
 		strcpy(nodename3, str);
-		dot = strstr(nodename3, ".");
+		dot = strchr(nodename3, '.');
 		if (dot)
 			len = dot-nodename3;
 		else
@@ -325,7 +363,6 @@ static int verify_nodename(int cd, char *nodename)
 		return -1;
 
 	for (ifa = ifa_list; ifa; ifa = ifa->ifa_next) {
-
 		/* Restore this */
 		strcpy(nodename2, nodename);
 		sa = ifa->ifa_addr;
@@ -338,8 +375,11 @@ static int verify_nodename(int cd, char *nodename)
 			goto out;
 
 		str = NULL;
-		memset(path, 0, 256);
-		sprintf(path, NODE_NAME_PATH_BYNAME, nodename2);
+		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename2);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			goto out;
+		}
 
 		error = ccs_get(cd, path, &str);
 		if (!error) {
@@ -350,14 +390,17 @@ static int verify_nodename(int cd, char *nodename)
 
 		/* truncate this name and try again */
 
-		dot = strstr(nodename2, ".");
+		dot = strchr(nodename2, '.');
 		if (!dot)
 			continue;
 		*dot = '\0';
 
 		str = NULL;
-		memset(path, 0, 256);
-		sprintf(path, NODE_NAME_PATH_BYNAME, nodename2);
+		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename2);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			goto out;
+		}
 
 		error = ccs_get(cd, path, &str);
 		if (!error) {
@@ -373,8 +416,11 @@ static int verify_nodename(int cd, char *nodename)
 			goto out;
 
 		str = NULL;
-		memset(path, 0, 256);
-		sprintf(path, NODE_NAME_PATH_BYNAME, nodename2);
+		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename2);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			goto out;
+		}
 
 		error = ccs_get(cd, path, &str);
 		if (!error) {
@@ -390,24 +436,83 @@ static int verify_nodename(int cd, char *nodename)
 	return error;
 }
 
+/* get any environment variable overrides */
+static int get_overrides()
+{
+	if (getenv("CMAN_CLUSTER_NAME")) {
+		strcpy(cluster_name, getenv("CMAN_CLUSTER_NAME"));
+		log_printf(LOG_INFO, "Using override cluster name %s\n", cluster_name);
+	}
+
+	nodename_env = getenv("CMAN_NODENAME");
+	if (nodename_env) {
+		log_printf(LOG_INFO, "Using override node name %s\n", nodename_env);
+	}
+
+	expected_votes = 0;
+	if (getenv("CMAN_EXPECTEDVOTES")) {
+		expected_votes = atoi(getenv("CMAN_EXPECTEDVOTES"));
+		if (expected_votes < 1) {
+			log_printf(LOG_ERR, "CMAN_EXPECTEDVOTES environment variable is invalid, ignoring");
+			expected_votes = 0;
+		}
+		else {
+			log_printf(LOG_INFO, "Using override expected votes %d\n", expected_votes);
+		}
+	}
+
+	/* optional port */
+	if (getenv("CMAN_IP_PORT")) {
+		portnums[0] = atoi(getenv("CMAN_IP_PORT"));
+		log_printf(LOG_INFO, "Using override IP port %d\n", portnums[0]);
+	}
+
+	/* optional security key filename */
+	if (getenv("CMAN_KEYFILE")) {
+		key_filename = strdup(getenv("CMAN_KEYFILE"));
+		if (key_filename == NULL)
+			return -ENOMEM;
+	}
+
+	/* find our own number of votes */
+	if (getenv("CMAN_VOTES")) {
+		votes = atoi(getenv("CMAN_VOTES"));
+		log_printf(LOG_INFO, "Using override votes %d\n", votes);
+	}
+
+	/* nodeid */
+	if (getenv("CMAN_NODEID")) {
+		nodeid = atoi(getenv("CMAN_NODEID"));
+		log_printf(LOG_INFO, "Using override nodeid %d\n", nodeid);
+	}
+
+	if (getenv("CMAN_MCAST_ADDR")) {
+		mcast_name = getenv("CMAN_MCAST_ADDR");
+		log_printf(LOG_INFO, "Using override multicast address %s\n", mcast_name);
+	}
+
+	if (getenv("CMAN_2NODE")) {
+		two_node = 1;
+		expected_votes = 1;
+		votes = 1;
+		log_printf(LOG_INFO, "Setting two_node mode from cman_tool\n");
+	}
+
+	return 0;
+}
 
 static int get_ccs_join_info(void)
 {
 	char path[MAX_PATH_LEN];
 	char nodename[MAX_CLUSTER_MEMBER_NAME_LEN+1];
-	char *str, *name, *cname = NULL;
+	char *str, *name;
 	int cd, error, i, vote_sum = 0, node_count = 0;
 	unsigned short port = 0;
 
 	/* Connect to ccsd */
-	if (getenv("CMAN_CLUSTER_NAME")) {
-		cname = getenv("CMAN_CLUSTER_NAME");
-		log_msg(LOG_INFO, "Using override cluster name %s\n", cname);
-	}
-
-	cd = ccs_force_connect(cname, 1);
+	cd = ccs_force_connect(cluster_name[0]?cluster_name:NULL, 1);
 	if (cd < 0) {
-		log_msg(LOG_ERR, "Error connecting to CCS");
+		log_printf(LOG_ERR, "Error connecting to CCS");
 		write_cman_pipe("Can't connect to CCSD");
 		return -ENOTCONN;
 	}
@@ -415,20 +520,29 @@ static int get_ccs_join_info(void)
 	/* Cluster name */
 	error = ccs_get(cd, CLUSTER_NAME_PATH, &str);
 	if (error) {
-		log_msg(LOG_ERR, "cannot find cluster name in config file");
+		log_printf(LOG_ERR, "cannot find cluster name in config file");
 		write_cman_pipe("Can't find cluster name in CCS");
-		return -ENOENT;
+		error = -ENOENT;
+		goto out;
 	}
 
-	if (cname) {
-		if (strcmp(cname, str)) {
-			log_msg(LOG_ERR, "cluster names not equal %s %s", cname, str);
+	if (cluster_name[0]) {
+		if (strcmp(cluster_name, str)) {
+			log_printf(LOG_ERR, "cluster names not equal %s %s", cluster_name, str);
 			write_cman_pipe("Cluster name in CCS does not match that passed to cman_tool");
-			return -ENOENT;
+			error = -ENOENT;
+			goto out;
 		}
-	} else {
-		strcpy(cluster_name, str);
 	}
+
+	if (strlen(str) >= sizeof(cluster_name)) {
+		free(str);
+		write_cman_pipe("Cluster name in CCS is too long");
+		error = -E2BIG;
+		goto out;
+	}
+
+	strcpy(cluster_name, str);
 	free(str);
 
 	error = ccs_get(cd, CLUSTER_ID_PATH, &str);
@@ -440,33 +554,50 @@ static int get_ccs_join_info(void)
 		cluster_id = generate_cluster_id(cluster_name);
 	}
 
+	error = ccs_get(cd, MAXQUEUED_PATH, &str);
+	if (!error) {
+		max_outstanding_messages = atoi(str);
+		free(str);
+	}
+
 	/* our nodename */
-	memset(nodename, 0, sizeof(nodename));
-
-	if (getenv("CMAN_NODENAME")) {
-		strcpy(nodename, getenv("CMAN_NODENAME"));
-		log_msg(LOG_INFO, "Using override node name %s\n", nodename);
-
-		sprintf(path, NODE_NAME_PATH_BYNAME, nodename);
+	if (nodename_env) {
+		int ret;
+		ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNAME, nodename);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			log_printf(LOG_ERR, "Overridden node name %s is too long", nodename);
+			write_cman_pipe("Overridden node name is too long");
+			error = -E2BIG;
+			goto out;
+		}
 
 		error = ccs_get(cd, path, &str);
 		if (!error) {
 			free(str);
-		}
-		else {
-			log_msg(LOG_ERR, "Overridden node name %s is not in CCS", nodename);
+		} else {
+			log_printf(LOG_ERR, "Overridden node name %s is not in CCS", nodename);
 			write_cman_pipe("Overridden node name is not in CCS");
-			return -ENOENT;
+			error = -ENOENT;
+			goto out;
 		}
-	}
-	else {
+	} else {
 		struct utsname utsname;
+
 		error = uname(&utsname);
 		if (error) {
-			log_msg(LOG_ERR, "cannot get node name, uname failed");
+			log_printf(LOG_ERR, "cannot get node name, uname failed");
 			write_cman_pipe("Can't determine local node name");
-			return -ENOENT;
+			error = -ENOENT;
+			goto out;
 		}
+
+		if (strlen(utsname.nodename) >= sizeof(nodename)) {
+			log_printf(LOG_ERR, "node name from uname is too long");
+			write_cman_pipe("Can't determine local node name");
+			error = -E2BIG;
+			goto out;
+		}
+
 		strcpy(nodename, utsname.nodename);
 	}
 
@@ -474,31 +605,30 @@ static int get_ccs_join_info(void)
 	/* Find our nodename in cluster.conf */
 	error = verify_nodename(cd, nodename);
 	if (error) {
-		log_msg(LOG_ERR, "local node name \"%s\" not found in cluster.conf",
+		log_printf(LOG_ERR, "local node name \"%s\" not found in the configuration",
 			nodename);
-		write_cman_pipe("Can't find local node name in cluster.conf");
-		return -ENOENT;
+		write_cman_pipe("Can't find local node name in the configuration");
+		error = -ENOENT;
+		goto out;
 	}
-	nodenames[0] = strdup(nodename);
 
-	expected_votes = 0;
-	if (getenv("CMAN_EXPECTEDVOTES")) {
-		expected_votes = atoi(getenv("CMAN_EXPECTEDVOTES"));
-		if (expected_votes < 1) {
-			log_msg(LOG_ERR, "CMAN_EXPECTEDVOTES environment variable is invalid, ignoring");
-			expected_votes = 0;
-		}
-		else {
-			log_msg(LOG_INFO, "Using override expected votes %d\n", expected_votes);
-		}
+	nodenames[0] = strdup(nodename);
+	if (nodenames[0] == NULL) {
+		error = -ENOMEM;
+		goto out;
 	}
 
 	/* Sum node votes for expected */
 	if (expected_votes == 0) {
 		for (i = 1; ; i++) {
+			int ret;
+
 			name = NULL;
-			memset(path, 0, MAX_PATH_LEN);
-			sprintf(path, NODEI_NAME_PATH, i);
+			ret = snprintf(path, sizeof(path), NODE_NAME_PATH_BYNUM, i);
+			if (ret < 0 || (size_t) ret >= sizeof(path)) {
+				error = -E2BIG;
+				break;
+			}
 
 			error = ccs_get(cd, path, &name);
 			if (error || !name)
@@ -506,18 +636,23 @@ static int get_ccs_join_info(void)
 
 			node_count++;
 
-			memset(path, 0, MAX_PATH_LEN);
-			sprintf(path, NODE_VOTES_PATH, name);
+			ret = snprintf(path, sizeof(path), NODE_VOTES_PATH, name);
 			free(name);
 
+			if (ret < 0 || (size_t) ret >= sizeof(path)) {
+				error = -E2BIG;
+				break;
+			}
+
 			error = ccs_get(cd, path, &str);
 			if (error)
 				vote_sum++;
 			else {
 				if (atoi(str) < 0) {
-					log_msg(LOG_ERR, "negative votes not allowed");
+					log_printf(LOG_ERR, "negative votes not allowed");
 					write_cman_pipe("Found negative votes for this node in CCS");
-					return -EINVAL;
+					error = -EINVAL;
+					goto out;
 				}
 				vote_sum += atoi(str);
 				free(str);
@@ -534,13 +669,7 @@ static int get_ccs_join_info(void)
 			expected_votes = vote_sum;
 	}
 
-	/* optional port */
-	if (getenv("CMAN_IP_PORT")) {
-		port = atoi(getenv("CMAN_IP_PORT"));
-		log_msg(LOG_INFO, "Using override IP port %d\n", port);
-	}
-
-	if (!port) {
+	if (!portnums[0]) {
 		error = ccs_get(cd, PORT_PATH, &str);
 		if (!error) {
 			port = atoi(str);
@@ -548,75 +677,58 @@ static int get_ccs_join_info(void)
 		}
 		else
 			port = DEFAULT_PORT;
+		portnums[0] = port;
 	}
-	portnums[0] = port;
 
-	/* optional security key filename */
-	if (getenv("CMAN_KEYFILE")) {
-		key_filename = strdup(getenv("CMAN_KEYFILE"));
-	}
-	else {
+	if (!key_filename) {
 		error = ccs_get(cd, KEY_PATH, &str);
 		if (!error) {
 			key_filename = str;
 		}
 	}
 
-	/* find our own number of votes */
-	if (getenv("CMAN_VOTES")) {
-		votes = atoi(getenv("CMAN_VOTES"));
-		log_msg(LOG_INFO, "Using override votes %d\n", votes);
-	}
-
 	if (!votes) {
-		memset(path, 0, MAX_PATH_LEN);
-		sprintf(path, NODE_VOTES_PATH, nodename);
+		int ret = snprintf(path, sizeof(path), NODE_VOTES_PATH, nodename);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			log_printf(LOG_ERR, "unable to find votes for %s", nodename);
+			write_cman_pipe("Unable to find votes for node in CCS");
+			return -E2BIG;
+		}
 
 		error = ccs_get(cd, path, &str);
 		if (!error) {
 			int votestmp = atoi(str);
+			free(str);
 			if (votestmp < 0 || votestmp > 255) {
-				log_msg(LOG_ERR, "invalid votes value %d", votes);
+				log_printf(LOG_ERR, "invalid votes value %d", votestmp);
 				write_cman_pipe("Found invalid votes for node in CCS");
 				return -EINVAL;
 			}
 			votes = votestmp;
-			free(str);
 		}
 		else {
 			votes = 1;
 		}
 	}
 
-
-	/* nodeid */
-	if (getenv("CMAN_NODEID")) {
-		nodeid = atoi(getenv("CMAN_NODEID"));
-		log_msg(LOG_INFO, "Using override nodeid %d\n", nodeid);
-	}
-
 	if (!nodeid) {
-		memset(path, 0, MAX_PATH_LEN);
-		sprintf(path, NODE_NODEID_PATH, nodename);
+		int ret = snprintf(path, sizeof(path), NODE_NODEID_PATH, nodename);
 
-		error = ccs_get(cd, path, &str);
-		if (!error) {
-			nodeid = atoi(str);
-			free(str);
+		if (ret >= 0 && (size_t) ret < sizeof(path)) {
+			error = ccs_get(cd, path, &str);
+			if (!error) {
+				nodeid = atoi(str);
+				free(str);
+			}
 		}
 	}
 
 	if (!nodeid) {
-		log_msg(LOG_ERR, "No nodeid specified in cluster.conf");
+		log_printf(LOG_ERR, "No nodeid specified in configuration file");
 		write_cman_pipe("CCS does not have a nodeid for this node, run 'ccs_tool addnodeids' to fix");
 		return -EINVAL;
 	}
 
-	if (getenv("CMAN_MCAST_ADDR")) {
-		mcast_name = getenv("CMAN_MCAST_ADDR");
-		log_msg(LOG_INFO, "Using override multicast address %s\n", mcast_name);
-	}
-
 	/* Optional multicast name */
 	if (!mcast_name) {
 		error = ccs_get(cd, MCAST_ADDR_PATH, &str);
@@ -627,7 +739,7 @@ static int get_ccs_join_info(void)
 
 	if (!mcast_name) {
 		mcast_name = default_mcast(cluster_id);
-		log_msg(LOG_INFO, "Using default multicast address of %s\n", mcast_name);
+		log_printf(LOG_INFO, "Using default multicast address of %s\n", mcast_name);
 	}
 
 	mcast[0] = mcast_name;
@@ -635,35 +747,44 @@ static int get_ccs_join_info(void)
 	/* Get all alternative node names */
 	num_nodenames = 1;
 
-	memset(path, 0, MAX_PATH_LEN);
-
-
 	for (i = 1; ; i++) {
-		str = NULL;
-		sprintf(path, NODE_ALTNAMES_PATH, nodename, i);
+		int ret = snprintf(path, sizeof(path), NODE_ALTNAMES_PATH, nodename, i);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			break;
+		}
 
+		str = NULL;
 		error = ccs_get(cd, path, &str);
 		if (error || !str)
 			break;
 
 		nodenames[i] = str;
 
-		sprintf(path, NODE_ALTNAMES_PORT, nodename, i);
+		ret = snprintf(path, sizeof(path), NODE_ALTNAMES_PORT, nodename, i);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			break;
+		}
+
 		error = ccs_get(cd, path, &str);
 		if (error || !str) {
 			portnums[i] = portnums[0];
-		}
-		else {
+		} else {
 			portnums[i] = atoi(str);
 			free(str);
 		}
 
-		sprintf(path, NODE_ALTNAMES_MCAST, nodename, i);
+		ret = snprintf(path, sizeof(path), NODE_ALTNAMES_MCAST, nodename, i);
+		if (ret < 0 || (size_t) ret >= sizeof(path)) {
+			error = -E2BIG;
+			break;
+		}
+
 		error = ccs_get(cd, path, &str);
 		if (error || !str) {
 			mcast[i] = mcast_name;
-		}
-		else {
+		} else {
 			mcast[i] = str;
 		}
 
@@ -678,29 +799,106 @@ static int get_ccs_join_info(void)
 		free(str);
 		if (two_node) {
 			if (node_count != 2 || vote_sum != 2) {
-				log_msg(LOG_ERR, "the two-node option requires exactly two "
+				log_printf(LOG_ERR, "the two-node option requires exactly two "
 					"nodes with one vote each and expected "
 					"votes of 1 (node_count=%d vote_sum=%d)",
 					node_count, vote_sum);
 				write_cman_pipe("two_node set but there are more than 2 nodes");
-				return -EINVAL;
+				error = -EINVAL;
+				goto out;
 			}
 
 			if (votes != 1) {
-				log_msg(LOG_ERR, "the two-node option requires exactly two "
+				log_printf(LOG_ERR, "the two-node option requires exactly two "
 					"nodes with one vote each and expected "
 					"votes of 1 (votes=%d)", votes);
 				write_cman_pipe("two_node set but votes not set to 1");
-				return -EINVAL;
+				error = -EINVAL;
+				goto out;
 			}
 		}
 	}
 
+	error = 0;
+
+out:
 	ccs_disconnect(cd);
-	return 0;
+	return error;
 }
 
 
+/* If ccs is not available then use some defaults */
+static int noccs_defaults()
+{
+	/* Enforce key */
+	key_filename = NOCCS_KEY_FILENAME;
+
+	if (cluster_name[0] == '\0')
+		strcpy(cluster_name, DEFAULT_CLUSTER_NAME);
+
+	if (!cluster_id)
+		cluster_id = generate_cluster_id(cluster_name);
+
+	if (!nodename_env) {
+		int error;
+		struct utsname utsname;
+
+		error = uname(&utsname);
+		if (error) {
+			log_printf(LOG_ERR, "cannot get node name, uname failed");
+			write_cman_pipe("Can't determine local node name");
+			return -ENOENT;
+		}
+
+		nodename_env = utsname.nodename;
+	}
+	nodenames[0] = strdup(nodename_env);
+	num_nodenames = 1;
+
+	if (!mcast_name) {
+		mcast_name = default_mcast(cluster_id);
+		log_printf(LOG_INFO, "Using default multicast address of %s\n", mcast_name);
+	}
+	mcast[0] = mcast_name;
+
+	/* This will increase as nodes join the cluster */
+	if (!expected_votes)
+		expected_votes = 1;
+	if (!votes)
+		votes = 1;
+
+	if (!portnums[0])
+		portnums[0] = DEFAULT_PORT;
+
+	/* Invent a node ID */
+	if (!nodeid) {
+		struct addrinfo *ainfo;
+		struct addrinfo ahints;
+		int ret;
+
+		memset(&ahints, 0, sizeof(ahints));
+		ret = getaddrinfo(nodenames[0], NULL, &ahints, &ainfo);
+		if (ret) {
+			log_printf(LOG_ERR, "Can't determine address family of nodename %s\n", nodenames[0]);
+			write_cman_pipe("Can't determine address family of nodename");
+			return -EINVAL;
+		}
+
+		if (ainfo->ai_family == AF_INET) {
+			struct sockaddr_in *addr = (struct sockaddr_in *)ainfo->ai_addr;
+			memcpy(&nodeid, &addr->sin_addr, sizeof(int));
+		}
+		if (ainfo->ai_family == AF_INET6) {
+			struct sockaddr_in6 *addr = (struct sockaddr_in6 *)ainfo->ai_addr;
+			memcpy(&nodeid, &addr->sin6_addr.in6_u.u6_addr32[3], sizeof(int));
+		}
+		log_printf(LOG_INFO, "Our Node ID is %d\n", nodeid);
+		freeaddrinfo(ainfo);
+	}
+
+	return 0;
+}
+
 
 /* Read just the stuff we need to get started.
    This does what 'cman_tool join' used to to */
@@ -708,9 +906,13 @@ int read_ccs_config()
 {
 	int error;
 
-	error = get_ccs_join_info();
+	get_overrides();
+	if (getenv("CMAN_NOCCS"))
+		error = noccs_defaults();
+	else
+		error = get_ccs_join_info();
 	if (error) {
-		log_msg(LOG_ERR, "Error reading CCS info, cannot start");
+		log_printf(LOG_ERR, "Error reading configuration info, cannot start");
 		return error;
 	}
 
@@ -718,3 +920,4 @@ int read_ccs_config()
 
 	return error;
 }
+
