@@ -1,7 +1,7 @@
 /******************************************************************************
 *******************************************************************************
 **
-**  Copyright (C) 2005-2006 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2005-2008 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -60,8 +60,10 @@ volatile sig_atomic_t quit_threads=0;
 
 int num_connections = 0;
 poll_handle ais_poll_handle;
+uint32_t max_outstanding_messages = 128;
 
 static int process_client(poll_handle handle, int fd, int revent, void *data);
+static void remove_client(poll_handle handle, struct connection *con);
 
 /* Send it, or queue it for later if the socket is busy */
 static int send_reply_message(struct connection *con, struct sock_header *msg)
@@ -82,8 +84,17 @@ static int send_reply_message(struct connection *con, struct sock_header *msg)
 
 	if ((ret > 0 && ret != msg->length) ||
 	    (ret == -1 && errno == EAGAIN)) {
+		struct queued_reply *qm;
+
+		/* Have we exceeded the allowed number of queued messages ? */
+		if (con->num_write_msgs > max_outstanding_messages) {
+			P_DAEMON("Disconnecting. client has more that %d replies outstanding (%d)\n", max_outstanding_messages, con->num_write_msgs);
+			remove_client(ais_poll_handle, con);
+			return -1;
+		}
+
 		/* Queue it */
-		struct queued_reply *qm = malloc(sizeof(struct queued_reply) + msg->length);
+		qm = malloc(sizeof(struct queued_reply) + msg->length);
 		if (!qm)
 		{
 			perror("Error allocating queued message");
@@ -95,15 +106,19 @@ static int send_reply_message(struct connection *con, struct sock_header *msg)
 		else
 			qm->offset = 0;
 		list_add(&con->write_msgs, &qm->list);
-		P_DAEMON("queued last message\n");
+		con->num_write_msgs++;
+		P_DAEMON("queued last message, count is %d\n", con->num_write_msgs);
 		poll_dispatch_modify(ais_poll_handle, con->fd, POLLIN | POLLOUT, process_client);
 	}
 	return 0;
 }
 
-
 static void remove_client(poll_handle handle, struct connection *con)
 {
+	struct list *tmp, *qmh;
+	struct queued_reply *qm;
+	int msgs=0;
+
 	poll_dispatch_delete(handle, con->fd);
 	close(con->fd);
 	if (con->type == CON_CLIENT)
@@ -112,6 +127,15 @@ static void remove_client(poll_handle handle, struct connection *con)
 	unbind_con(con);
 	remove_barriers(con);
 
+	list_iterate_safe(qmh, tmp, &con->write_msgs) {
+		qm = list_item(qmh, struct queued_reply);
+
+		list_del(&qm->list);
+		free(qm);
+		msgs++;
+	}
+
+	P_DAEMON("Freed %d queued messages\n", msgs);
 	free(con);
 	num_connections--;
 }
@@ -132,6 +156,7 @@ static void send_queued_reply(struct connection *con)
 		{
 			list_del(&qm->list);
 			free(qm);
+			con->num_write_msgs--;
 		}
 		else
 		{
@@ -246,10 +271,16 @@ static int process_client(poll_handle handle, int fd, int revent, void *data)
 
 			buf += sizeof(struct sock_data_header);
 
+			if (dmsg->port > 255) {
+				send_status_return(con, msg->command, -EINVAL);
+				return 0;
+			}
+
 			if (dmsg->port)
 				port = dmsg->port;
 			else
 				port = con->port;
+
 			ret = comms_send_message(buf, msg->length - sizeof(struct sock_data_header),
 						 port, con->port,
 						 dmsg->nodeid,
@@ -317,6 +348,7 @@ static int process_rendezvous(poll_handle handle, int fd, int revent, void *data
 		newcon->type = con->type;
 		newcon->port = 0;
 		newcon->events = 0;
+		newcon->num_write_msgs = 0;
 		list_init(&newcon->write_msgs);
 		fcntl(client_fd, F_SETFL, fcntl(client_fd, F_GETFL, 0) | O_NONBLOCK);
 
