@@ -26,6 +26,8 @@
 #include "libdlm.h"
 #include "dlm_controld.h"
 
+#define LKM_IVMODE -1
+
 #define OPTION_STRING			"MhVvd:m:"
 
 #define OP_JOIN				1
@@ -367,14 +369,14 @@ void do_lockdump(char *name)
 			continue;
 		}
 
-		/* A hack because dlm-kernel doesn't set rqmode back to NL when
+		/* A hack because dlm-kernel doesn't set rqmode back to IV when
 		   a NOQUEUE convert fails, which means in a lockdump it looks
 		   like a granted lock is still converting since rqmode is not
-		   NL.  (does it make sense to include status in the output,
+		   IV.  (does it make sense to include status in the output,
 		   e.g. G,C,W?) */
 
 		if (status == DLM_LKSTS_GRANTED)
-			rqmode = LKM_NLMODE;
+			rqmode = LKM_IVMODE;
 
 		printf("id %08x gr %s rq %s pid %u master %d \"%s\"\n",
 			id, mode_str(grmode), mode_str(rqmode),
