@@ -2,7 +2,7 @@
 *******************************************************************************
 **
 **  Copyright (C) Sistina Software, Inc.  1997-2003  All rights reserved.
-**  Copyright (C) 2004 Red Hat, Inc.  All rights reserved.
+**  Copyright (C) 2004-2006 Red Hat, Inc.  All rights reserved.
 **
 **  This copyrighted material is made available to anyone wishing to use,
 **  modify, copy, or redistribute it subject to the terms and conditions
@@ -23,6 +23,9 @@ MAKE_MULT8(sizeof(struct gfs_ea_header) + \
 	   ((GFS_EA_IS_STUFFED(ea)) ? \
 	    GFS_EA_DATA_LEN(ea) : \
 	    (sizeof(uint64_t) * (ea)->ea_num_ptrs)))
+#define GFS_EA_STRLEN(ea) \
+((((ea)->ea_type == GFS_EATYPE_USR) ? 5 : 7) + \
+ (ea)->ea_name_len + 1)
 
 #define GFS_EA_IS_STUFFED(ea) (!(ea)->ea_num_ptrs)
 #define GFS_EA_IS_LAST(ea) ((ea)->ea_flags & GFS_EAFLAG_LAST)
