@@ -1241,7 +1241,7 @@ int main(int argc, char **argv)
       continue;
     case 'V':
       printf("%s %s (built %s %s)\n", argv[0],
-             GNBD_RELEASE_NAME, __DATE__, __TIME__);
+             RELEASE_VERSION, __DATE__, __TIME__);
       printf("%s\n", REDHAT_COPYRIGHT);
       return 0;
     case 'v':
